/*
 * This class is based on code from PlugableRendererDemo from JUNG framework 
 *
 * Copyright by Neuroph Project (c) 2004, the JUNG Project and the Regents of the University of
 * California All rights reserved.
 * 
 * This software is open-source under the BSD license; see either "license.txt"
 * or http://jung.sourceforge.net/license.txt for a description.
 * 
 * Authors: Danyel Fisher, Joshua O'Madadhain, Tom Nelson
 * Created on Nov 7, 2004
 */

// TODO: add Radial Layout, Tree Layout, Static layout, layout combo, editor
package org.neuroph.easyneurons;

import java.awt.BasicStroke;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GradientPaint;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Paint;
import java.awt.Shape;
import java.awt.Stroke;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.geom.Point2D;
import java.lang.reflect.Constructor;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.swing.AbstractAction;
import javax.swing.AbstractButton;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.ButtonGroup;
import javax.swing.DefaultListCellRenderer;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JRadioButton;

import org.apache.commons.collections.Predicate;
import org.neuroph.core.Connection;
import org.neuroph.core.Layer;
import org.neuroph.core.NeuralNetwork;
import org.neuroph.core.Neuron;
import org.neuroph.easyneurons.graph.ConnectionEdge;
import org.neuroph.easyneurons.graph.NNGraph;
import org.neuroph.easyneurons.graph.NeuronVertex;

import edu.uci.ics.jung.graph.DirectedEdge;
import edu.uci.ics.jung.graph.Edge;
import edu.uci.ics.jung.graph.Graph;
import edu.uci.ics.jung.graph.UndirectedEdge;
import edu.uci.ics.jung.graph.Vertex;
import edu.uci.ics.jung.graph.decorators.AbstractVertexShapeFunction;
import edu.uci.ics.jung.graph.decorators.ConstantEdgeStringer;
import edu.uci.ics.jung.graph.decorators.ConstantVertexStringer;
import edu.uci.ics.jung.graph.decorators.DefaultToolTipFunction;
import edu.uci.ics.jung.graph.decorators.EdgeFontFunction;
import edu.uci.ics.jung.graph.decorators.EdgePaintFunction;
import edu.uci.ics.jung.graph.decorators.EdgeShape;
import edu.uci.ics.jung.graph.decorators.EdgeStringer;
import edu.uci.ics.jung.graph.decorators.EdgeStrokeFunction;
import edu.uci.ics.jung.graph.decorators.GradientEdgePaintFunction;
import edu.uci.ics.jung.graph.decorators.NumberEdgeValue;
import edu.uci.ics.jung.graph.decorators.NumberEdgeValueStringer;
import edu.uci.ics.jung.graph.decorators.NumberVertexValue;
import edu.uci.ics.jung.graph.decorators.NumberVertexValueStringer;
import edu.uci.ics.jung.graph.decorators.PickableEdgePaintFunction;
import edu.uci.ics.jung.graph.decorators.UserDatumNumberEdgeValue;
import edu.uci.ics.jung.graph.decorators.UserDatumNumberVertexValue;
import edu.uci.ics.jung.graph.decorators.VertexAspectRatioFunction;
import edu.uci.ics.jung.graph.decorators.VertexFontFunction;
import edu.uci.ics.jung.graph.decorators.VertexPaintFunction;
import edu.uci.ics.jung.graph.decorators.VertexSizeFunction;
import edu.uci.ics.jung.graph.decorators.VertexStringer;
import edu.uci.ics.jung.graph.decorators.VertexStrokeFunction;
import edu.uci.ics.jung.graph.predicates.SelfLoopEdgePredicate;
import edu.uci.ics.jung.random.generators.BarabasiAlbertGenerator;
import edu.uci.ics.jung.utils.MutableDouble;
import edu.uci.ics.jung.visualization.DefaultSettableVertexLocationFunction;
import edu.uci.ics.jung.visualization.FRLayout;
import edu.uci.ics.jung.visualization.GraphZoomScrollPane;
import edu.uci.ics.jung.visualization.HasGraphLayout;
import edu.uci.ics.jung.visualization.Layout;
import edu.uci.ics.jung.visualization.PickSupport;
import edu.uci.ics.jung.visualization.PickedInfo;
import edu.uci.ics.jung.visualization.PickedState;
import edu.uci.ics.jung.visualization.PluggableRenderer;
import edu.uci.ics.jung.visualization.ShapePickSupport;
import edu.uci.ics.jung.visualization.VisualizationViewer;
import edu.uci.ics.jung.visualization.contrib.CircleLayout;
import edu.uci.ics.jung.visualization.contrib.KKLayout;
import edu.uci.ics.jung.visualization.control.AbstractPopupGraphMousePlugin;
import edu.uci.ics.jung.visualization.control.CrossoverScalingControl;
import edu.uci.ics.jung.visualization.control.EditingModalGraphMouse;
import edu.uci.ics.jung.visualization.control.EditingPopupGraphMousePlugin;
import edu.uci.ics.jung.visualization.control.ModalGraphMouse;
import edu.uci.ics.jung.visualization.control.ScalingControl;
import edu.uci.ics.jung.visualization.transform.LayoutTransformer;
import edu.uci.ics.jung.visualization.transform.Transformer;

/**
 * @author Zoran Sevarac
 */
public class NeuralGraphRenderer implements ActionListener {
	protected JCheckBox v_color;
	protected JCheckBox e_color;
	protected JCheckBox v_stroke;
	protected JCheckBox e_uarrow_pred;
	protected JCheckBox e_darrow_pred;
	protected JCheckBox v_shape;
	protected JCheckBox v_size;
	protected JCheckBox v_aspect;
	protected JCheckBox v_labels;
	protected JRadioButton e_line;
	protected JRadioButton e_bent;
	protected JRadioButton e_wedge;
	protected JRadioButton e_quad;
	protected JRadioButton e_cubic;
	protected JCheckBox e_labels;
	protected JCheckBox font;
	protected JCheckBox e_show_d;
	protected JCheckBox e_show_u;
	protected JCheckBox v_small;
	protected JCheckBox zoom_at_mouse;
	protected JCheckBox fill_edges;

	protected JRadioButton layout_FR;
	protected JRadioButton layout_KK;
	protected JRadioButton layout_Circle;
	protected JComboBox layoutCombo;

	protected JRadioButton no_gradient;
	// protected JRadioButton gradient_absolute;
	protected JRadioButton gradient_relative;

	protected static final int GRADIENT_NONE = 0;
	protected static final int GRADIENT_RELATIVE = 1;
	// protected static final int GRADIENT_ABSOLUTE = 2;
	protected static int gradient_level = GRADIENT_NONE;

	protected PluggableRenderer pr;
	protected SeedColor vcf;
	protected EdgeWeightStrokeFunction ewcs;
	protected VertexStrokeHighlight vsh;
	protected FontHandler ff;
	protected VertexShapeSizeAspect vssa;
	protected DirectionDisplayPredicate show_edge;
	protected DirectionDisplayPredicate show_arrow;
	protected VertexDisplayPredicate show_vertex;
	protected GradientPickedEdgePaintFunction edgePaint;

	protected DefaultSettableVertexLocationFunction vertexLocations;

	// stringers are used for neuron activation and connection weight labels
	protected VertexStringer vs;
	protected VertexStringer vs_none;
	protected EdgeStringer es;
	protected EdgeStringer es_none;

	protected final static Object ACTIVATION_KEY = "neuron_activation";
	protected final static Object WEIGHT_KEY = "connection_weight";

	protected final static Object TRANSPARENCY = "transparency";

	protected NumberEdgeValue connection_weight = new UserDatumNumberEdgeValue(
			"connection_weight");
	protected NumberVertexValue neuron_activation = new UserDatumNumberVertexValue(
			"neuron_activation");
	protected NumberVertexValue transparency = new UserDatumNumberVertexValue(
			TRANSPARENCY);

	protected VisualizationViewer vv;
	// protected DefaultModalGraphMouse graphMouse;
	protected EditingModalGraphMouse graphMouse;

	protected Transformer affineTransformer;

	protected Graph graph;
	protected NeuralNetwork neuralNet;
	protected JPanel graphPanel;

	protected Layout layout;

	public NeuralGraphRenderer(NeuralNetwork neuralNet) {
		this.neuralNet = neuralNet;
		pr = new PluggableRenderer();

		v_labels = new JCheckBox("show activation levels");
		v_labels.addActionListener(this);

		vs_none = new ConstantVertexStringer(null);
		pr.setVertexStringer(vs_none);

		e_labels = new JCheckBox("show weights");
		e_labels.addActionListener(this);

		es_none = new ConstantEdgeStringer(null);
		pr.setEdgeStringer(es_none);

		graph = createGraph();
		layout = new FRLayout(graph);
		vv = new VisualizationViewer(layout, pr);

		vertexLocations = new DefaultSettableVertexLocationFunction();

		createGraphPanel();
	}

	public JPanel getGraphPanel() {
		return this.graphPanel;
	}

	public void createGraphPanel() {
		graph = createGraph();

		layout = new FRLayout(graph);
		vv = new VisualizationViewer(layout, pr);
		// add Shape based pick support
		vv.setPickSupport(new ShapePickSupport());
		PickedState picked_state = vv.getPickedState();

		affineTransformer = vv.getLayoutTransformer();

		// create decorators
		vcf = new SeedColor(picked_state);
		ewcs = new EdgeWeightStrokeFunction(connection_weight);
		vsh = new VertexStrokeHighlight(picked_state);
		ff = new FontHandler();

		vssa = new VertexShapeSizeAspect(neuron_activation);
		show_edge = new DirectionDisplayPredicate(true, true);
		show_arrow = new DirectionDisplayPredicate(true, false);
		show_vertex = new VertexDisplayPredicate(false);

		// uses a gradient edge if unpicked, otherwise uses picked selection
		edgePaint = new GradientPickedEdgePaintFunction(
				new PickableEdgePaintFunction(picked_state, Color.black,
						Color.cyan), vv, vv, picked_state);

		pr.setVertexPaintFunction(vcf);
		pr.setVertexStrokeFunction(vsh);

		pr.setVertexFontFunction(ff);
		pr.setVertexShapeFunction(vssa);
		pr.setVertexIncludePredicate(show_vertex);

		pr.setEdgePaintFunction(edgePaint);
		pr.setEdgeStringer(es_none);
		pr.setEdgeFontFunction(ff);
		pr.setEdgeStrokeFunction(ewcs);
		pr.setEdgeIncludePredicate(show_edge);
		pr.setEdgeShapeFunction(new EdgeShape.Line());
		pr.setEdgeArrowPredicate(show_arrow);
		this.graphPanel = new JPanel();
		graphPanel.setLayout(new BorderLayout());

		vv.setBackground(Color.white);
		GraphZoomScrollPane scrollPane = new GraphZoomScrollPane(vv);
		scrollPane.setPreferredSize(new Dimension(930, 400));
		graphPanel.add(scrollPane);
		graphMouse = new EditingModalGraphMouse();
		vv.setGraphMouse(graphMouse);
		graphMouse.add(new EditingPopupGraphMousePlugin(vertexLocations));
		graphMouse.setMode(ModalGraphMouse.Mode.TRANSFORMING);
		// graphMouse.add(new PopupGraphMousePlugin());

		addControls(graphPanel);
		vssa.setScaling(true);

		vv.setToolTipFunction(new VoltageTips());
		vv
				.setToolTipText("<html><center>Use the mouse wheel to zoom<p>Click and Drag the mouse to pan<p>Shift-click and Drag to Rotate</center></html>");

	}

	/**
	 * Generates a mixed-mode random graph, runs VoltageRanker on it, and
	 * returns the resultant graph.
	 */
	public Graph createGraph() {
		// NeuralNetwork nnet=NeuralNetworkFactory.createMLPerceptron("3 4 5 8",
		// TransferFunctionType.SIGMOID);

		NNGraph g = new NNGraph();

		vs = new NumberVertexValueStringer(neuron_activation);
		es = new NumberEdgeValueStringer(connection_weight);

		Iterator<Layer> li = this.neuralNet.getLayersIterator();
		while (li.hasNext()) {
			Layer l = li.next();
			Iterator<Neuron> ni = l.getNeuronsIterator();
			while (ni.hasNext()) {
				Neuron n = ni.next();
				NeuronVertex v = new NeuronVertex(n);
				// v.addUserDatum(new Double(n.getOutput()),
				// "neuron_activation", UserData.SHARED);
				neuron_activation.setNumber(v, new Double(n.getOutput()));
				g.addVertex(v);

				Iterator<Connection> ci = n.getInputConnections().iterator();
				while (ci.hasNext()) {
					Connection connection = ci.next();
					Neuron fromNeuron = connection.getConnectedNeuron();
					NeuronVertex fromVertex = g.getVertexFor(fromNeuron);
					ConnectionEdge edge = new ConnectionEdge(fromVertex, v);
					connection_weight.setNumber(edge, new Double(connection
							.getWeight().getValue()));
					// edge.addUserDatum(new
					// Double(connection.getWeight().getValue()),
					// "connection_weight", UserData.SHARED);
					g.addEdge(edge);
				}

			}
		}

		// Create some MyNode objects to use as vertices
		// SparseVertex n1 = new SparseVertex();
		// SparseVertex n2 = new SparseVertex();
		// SparseVertex n3 = new SparseVertex();
		// SparseVertex n4 = new SparseVertex();
		// SparseVertex n5 = new SparseVertex();
		// graph.addVertex(n1);
		// graph.addVertex(n2);
		// graph.addVertex(n3);
		// graph.addVertex(n4);
		// graph.addVertex(n5);
		// graph.addEdge(new UndirectedSparseEdge(n1, n2)); // This method
		// graph.addEdge(new DirectedSparseEdge(n2, n3));
		// graph.addEdge(new DirectedSparseEdge(n3, n4));
		// graph.addEdge(new DirectedSparseEdge(n4, n5)); // or we can use

		return g;
	}

	public void updateGraph() {
		Iterator<?> iterator = graph.getVertices().iterator();
		while (iterator.hasNext()) {
			NeuronVertex v = (NeuronVertex) iterator.next();
			neuron_activation.setNumber(v,
					new Double(v.getNeuron().getOutput()));
		}

		iterator = graph.getEdges().iterator();
		while (iterator.hasNext()) {
			ConnectionEdge edge = (ConnectionEdge) iterator.next();
			connection_weight.setNumber(edge, new Double(edge.getConnection()
					.getWeight().getValue()));
		}
	}

	/**
	 * @param jp
	 *            panel to which controls will be added
	 */
	protected void addControls(final JPanel jp) {
		final JPanel control_panel = new JPanel();
		jp.add(control_panel, BorderLayout.EAST);

		control_panel.setLayout(new GridBagLayout());

		java.awt.GridBagConstraints gridBagConstraints = new java.awt.GridBagConstraints();
		gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
		gridBagConstraints.insets = new java.awt.Insets(5, 5, 0, 5);
		gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;

		// control_panel.setBackground(new Color(00, 255, 00));

		final Box vertex_panel = Box.createVerticalBox();
		vertex_panel.setBorder(BorderFactory.createTitledBorder("Neurons"));
		final Box edge_panel = Box.createVerticalBox();
		edge_panel.setBorder(BorderFactory.createTitledBorder("Connections"));

		final Box edges_and_vertex_panel = Box.createVerticalBox();
		// Box zoom_and_mode_panel = Box.createVerticalBox();
		// zoom_and_mode_panel.setBorder(BorderFactory.createTitledBorder("Other controls"));
		final Box layout_panel = Box.createVerticalBox();
		layout_panel.setBorder(BorderFactory.createTitledBorder("Layout"));
		// JPanel zoom_and_mode_panel = new JPanel();
		// zoom_and_mode_panel.setLayout(new GridBagLayout() );
		// //GridLayout(3,1)
		// zoom_and_mode_panel.setBackground(new Color(0,255,0));

		edges_and_vertex_panel.add(vertex_panel);
		edges_and_vertex_panel.add(edge_panel);
		// edges_and_vertex_panel.add(layout_panel);

		gridBagConstraints.gridx = 0;
		gridBagConstraints.gridy = 0;
		control_panel.add(edges_and_vertex_panel, gridBagConstraints);

		gridBagConstraints.gridx = 0;
		gridBagConstraints.gridy = 1;
		control_panel.add(layout_panel, gridBagConstraints);

		// originalni layout
		// control_panel.add(vertex_panel, BorderLayout.WEST);
		// control_panel.add(edge_panel, BorderLayout.EAST);
		// control_panel.add(edges_and_vertex_panel, BorderLayout.CENTER);

		// set up vertex controls
		v_color = new JCheckBox("vertex seed coloring");
		v_color.addActionListener(this);
		v_stroke = new JCheckBox("<html>stroke highlight selected</html>");
		v_stroke.addActionListener(this);

		v_shape = new JCheckBox("vertex degree shapes");
		v_shape.addActionListener(this);
		v_size = new JCheckBox("activation size");
		v_size.addActionListener(this);
		v_size.setSelected(true);
		v_aspect = new JCheckBox("vertex degree ratio stretch");
		v_aspect.addActionListener(this);
		v_small = new JCheckBox("filter vertices of degree < "
				+ VertexDisplayPredicate.MIN_DEGREE);
		v_small.addActionListener(this);

		// vertex_panel.add(v_color);

		vertex_panel.add(v_labels);
		// vertex_panel.add(v_shape);
		vertex_panel.add(v_size);
		// vertex_panel.add(v_aspect);
		// vertex_panel.add(v_small);
		vertex_panel.add(v_stroke);

		// set up edge controls
		// JPanel gradient_panel = new JPanel(new GridLayout(1, 0));
		// gradient_panel.setBorder(BorderFactory.createTitledBorder("Edge paint"));
		// no_gradient = new JRadioButton("Solid color");
		// no_gradient.addActionListener(this);
		// no_gradient.setSelected(true);
		// // gradient_absolute = new JRadioButton("Absolute gradient");
		// // gradient_absolute.addActionListener(this);
		// gradient_relative = new JRadioButton("Gradient");
		// gradient_relative.addActionListener(this);
		// ButtonGroup bg_grad = new ButtonGroup();
		// bg_grad.add(no_gradient);
		// bg_grad.add(gradient_relative);
		// //bg_grad.add(gradient_absolute);
		// gradient_panel.add(no_gradient);
		// //gradientGrid.add(gradient_absolute);
		// gradient_panel.add(gradient_relative);

		JPanel shape_panel = new JPanel(new GridLayout(3, 2));
		shape_panel.setBorder(BorderFactory
				.createTitledBorder("Connection shape"));
		e_line = new JRadioButton("line");
		e_line.addActionListener(this);
		e_line.setSelected(true);
		// e_bent = new JRadioButton("bent line");
		// e_bent.addActionListener(this);
		e_wedge = new JRadioButton("wedge");
		e_wedge.addActionListener(this);
		e_quad = new JRadioButton("quad curve");
		e_quad.addActionListener(this);
		e_cubic = new JRadioButton("cubic curve");
		e_cubic.addActionListener(this);
		ButtonGroup bg_shape = new ButtonGroup();
		bg_shape.add(e_line);
		// bg.add(e_bent);
		bg_shape.add(e_wedge);
		bg_shape.add(e_quad);
		bg_shape.add(e_cubic);
		shape_panel.add(e_line);
		// shape_panel.add(e_bent);
		// shape_panel.add(e_wedge);
		shape_panel.add(e_quad);
		shape_panel.add(e_cubic);
		// fill_edges = new JCheckBox("fill edge shapes");
		// fill_edges.setSelected(false);
		// fill_edges.addActionListener(this);
		// shape_panel.add(fill_edges);
		shape_panel.setOpaque(true);
		e_color = new JCheckBox("weight highlighting");
		e_color.addActionListener(this);
		e_uarrow_pred = new JCheckBox("undirected");
		e_uarrow_pred.addActionListener(this);
		e_darrow_pred = new JCheckBox("directed");
		e_darrow_pred.addActionListener(this);
		e_darrow_pred.setSelected(true);
		// JPanel arrow_panel = new JPanel(new GridLayout(1,0));
		// arrow_panel.setBorder(BorderFactory.createTitledBorder("Show arrows"));
		// arrow_panel.add(e_uarrow_pred);
		// arrow_panel.add(e_darrow_pred);

		e_show_d = new JCheckBox("directed");
		e_show_d.addActionListener(this);
		e_show_d.setSelected(true);
		e_show_u = new JCheckBox("undirected");
		e_show_u.addActionListener(this);
		e_show_u.setSelected(true);
		JPanel show_edge_panel = new JPanel(new GridLayout(1, 0));
		show_edge_panel.setBorder(BorderFactory
				.createTitledBorder("Show edges"));
		show_edge_panel.add(e_show_u);
		show_edge_panel.add(e_show_d);

		shape_panel.setAlignmentX(Component.LEFT_ALIGNMENT);
		edge_panel.add(shape_panel);
		// gradient_panel.setAlignmentX(Component.LEFT_ALIGNMENT);
		// edge_panel.add(gradient_panel);
		// show_edge_panel.setAlignmentX(Component.LEFT_ALIGNMENT);
		// edge_panel.add(show_edge_panel);
		// arrow_panel.setAlignmentX(Component.LEFT_ALIGNMENT);
		// edge_panel.add(arrow_panel);

		e_color.setAlignmentX(Component.LEFT_ALIGNMENT);
		edge_panel.add(e_color);
		e_labels.setAlignmentX(Component.LEFT_ALIGNMENT);
		edge_panel.add(e_labels);

		Class<?>[] combos = getLayoutOptions();
		final JComboBox layoutCombo = new JComboBox(combos);
		// use a renderer to shorten the layout name presentation
		layoutCombo.setRenderer(new DefaultListCellRenderer() {
			public Component getListCellRendererComponent(JList list,
					Object value, int index, boolean isSelected,
					boolean cellHasFocus) {
				String valueString = value.toString();
				valueString = valueString.substring(valueString
						.lastIndexOf('.') + 1);
				return super.getListCellRendererComponent(list, valueString,
						index, isSelected, cellHasFocus);
			}
		});
		layoutCombo.addActionListener(new LayoutChooser(layoutCombo, vv));
		layoutCombo.setSelectedItem(FRLayout.class);
		layout_panel.add(layoutCombo);

		// set up zoom controls
		zoom_at_mouse = new JCheckBox(
				"<html><center>zoom at mouse<p>(wheel only)</center></html>");
		zoom_at_mouse.addActionListener(this);

		final ScalingControl scaler = new CrossoverScalingControl();

		JButton plus = new JButton("+");
		plus.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				scaler.scale(vv, 1.1f, vv.getCenter());
			}
		});
		JButton minus = new JButton("-");
		minus.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				scaler.scale(vv, 1 / 1.1f, vv.getCenter());
			}
		});

		JPanel zoomPanel = new JPanel();
		zoomPanel.setAlignmentX(Component.RIGHT_ALIGNMENT);
		zoomPanel.setBorder(BorderFactory.createTitledBorder("Zoom"));
		zoomPanel.add(plus);
		zoomPanel.add(minus);

		gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
		gridBagConstraints.insets = new java.awt.Insets(5, 5, 0, 5);
		gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;

		gridBagConstraints.gridx = 0;
		gridBagConstraints.gridy = 0;

		JComboBox modeBox = graphMouse.getModeComboBox();
		modeBox.setAlignmentX(Component.RIGHT_ALIGNMENT);

		JPanel modePanel = new JPanel(new BorderLayout()) {
			@Override
			public Dimension getMaximumSize() {
				return getPreferredSize();
			}
		};
		modePanel.setBorder(BorderFactory.createTitledBorder("Mouse Mode"));
		modePanel.add(modeBox);

		gridBagConstraints.gridx = 0;
		gridBagConstraints.gridy = 2;
		control_panel.add(modePanel, gridBagConstraints);

		gridBagConstraints.gridx = 0;
		gridBagConstraints.gridy = 3;
		control_panel.add(zoomPanel, gridBagConstraints);

		// add font control to center panel
		font = new JCheckBox("bold text");
		font.addActionListener(this);
		gridBagConstraints.gridx = 0;
		gridBagConstraints.gridy = 4;
		control_panel.add(font, gridBagConstraints);

	}

	public void actionPerformed(ActionEvent e) {
		AbstractButton source = (AbstractButton) e.getSource();
		if (source == v_color) {
			vcf.setSeedColoring(source.isSelected());
		} else if (source == e_color) {
			ewcs.setWeighted(source.isSelected());
		} else if (source == v_stroke) {
			vsh.setHighlight(source.isSelected());
		} else if (source == v_labels) {
			if (source.isSelected())
				pr.setVertexStringer(vs);
			else
				pr.setVertexStringer(vs_none);
		} else if (source == e_labels) {
			if (source.isSelected())
				pr.setEdgeStringer(es);
			else
				pr.setEdgeStringer(es_none);
		} else if (source == e_uarrow_pred) {
			show_arrow.showUndirected(source.isSelected());
		} else if (source == e_darrow_pred) {
			show_arrow.showDirected(source.isSelected());
		} else if (source == font) {
			ff.setBold(source.isSelected());
		} else if (source == v_shape) {
			vssa.useFunnyShapes(source.isSelected());
		} else if (source == v_size) {
			vssa.setScaling(source.isSelected());
		} else if (source == v_aspect) {
			vssa.setStretching(source.isSelected());
		} else if (source == e_line) {
			if (source.isSelected()) {
				pr.setEdgeShapeFunction(new EdgeShape.Line());
			}
		} else if (source == e_wedge) {
			if (source.isSelected())
				pr.setEdgeShapeFunction(new EdgeShape.Wedge(10));
		}
		// else if (source == e_bent)
		// {
		// if(source.isSelected())
		// {
		// pr.setEdgeShapeFunction(new EdgeShape.BentLine());
		// }
		// }
		else if (source == e_quad) {
			if (source.isSelected()) {
				pr.setEdgeShapeFunction(new EdgeShape.QuadCurve());
			}
		} else if (source == e_cubic) {
			if (source.isSelected()) {
				pr.setEdgeShapeFunction(new EdgeShape.CubicCurve());
			}
		} else if (source == e_show_d) {
			show_edge.showDirected(source.isSelected());
		} else if (source == e_show_u) {
			show_edge.showUndirected(source.isSelected());
		} else if (source == layout_FR) {
			vv.setGraphLayout(new FRLayout(this.graph));
			vv.restart();
		} else if (source == layout_Circle) {
			vv.setGraphLayout(new CircleLayout(this.graph));
			vv.restart();
		} else if (source == layout_KK) {
			vv.setGraphLayout(new KKLayout(this.graph));
			vv.restart();
		} else if (source == v_small) {
			show_vertex.filterSmall(source.isSelected());
		} else if (source == zoom_at_mouse) {
			graphMouse.setZoomAtMouse(source.isSelected());
		} else if (source == no_gradient) {
			if (source.isSelected()) {
				gradient_level = GRADIENT_NONE;
			}
			// } else if (source == gradient_absolute) {
			// if (source.isSelected()) {
			// gradient_level = GRADIENT_ABSOLUTE;
			// }
		} else if (source == gradient_relative) {
			if (source.isSelected()) {
				gradient_level = GRADIENT_RELATIVE;
			}
		} else if (source == fill_edges) {
			edgePaint.useFill(source.isSelected());
		} else if (source == layout_FR) {
			// pogledaj primer ShowLayouts pa premaa njemu napravi combo box sa
			// izborom layouta
			// edgePaint.useFill(source.isSelected());
			// vv.setLayout(new FRLayout(this.graph));
		} else if (source == layout_KK) {
			// edgePaint.useFill(source.isSelected());
		} else if (source == layout_Circle) {
			// edgePaint.useFill(source.isSelected());
		}

		vv.repaint();
	}

	public void update() {
		updateGraph(); // updates only activations, not structure
	}

	private static Class<?>[] getLayoutOptions() {
		List<Class<?>> layouts = new ArrayList<Class<?>>();
		layouts.add(KKLayout.class);
		layouts.add(FRLayout.class);
		layouts.add(CircleLayout.class);
		return layouts.toArray(new Class[0]);
	}

	/**
	 * 
	 * @author danyelf
	 */

	private class LayoutChooser implements ActionListener {
		private final JComboBox jcb;
		private final VisualizationViewer vv;

		private LayoutChooser(JComboBox jcb, VisualizationViewer vv) {
			super();
			this.jcb = jcb;
			this.vv = vv;
		}

		public void actionPerformed(ActionEvent arg0) {
			Object[] constructorArgs = { graph };

			Class<?> layoutC = (Class<?>) jcb.getSelectedItem();
			Class<?> lay = layoutC;
			try {
				Constructor<?> constructor = lay
						.getConstructor(new Class[] { Graph.class });
				Object o = constructor.newInstance(constructorArgs);

				Layout l = (Layout) o;
				vv.stop();
				vv.setGraphLayout(l, false);
				vv.restart();
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	}

	private final class SeedColor implements VertexPaintFunction {
		protected PickedInfo pi;
		protected final static float dark_value = 0.8f;
		protected final static float light_value = 0.2f;
		protected boolean seed_coloring;

		public SeedColor(PickedInfo pi) {
			this.pi = pi;
			seed_coloring = false;
		}

		public void setSeedColoring(boolean b) {
			this.seed_coloring = b;
		}

		public Paint getDrawPaint(Vertex v) {
			return Color.BLACK;
		}

		public Paint getFillPaint(Vertex v) {
			// float alpha = transparency.getNumber(v).floatValue();
			if (pi.isPicked(v)) {
				return new Color(1f, 1f, 0);
			} else {
				if (seed_coloring
						&& v.containsUserDatumKey(BarabasiAlbertGenerator.SEED)) {
					Color dark = new Color(0, 0, dark_value);
					Color light = new Color(0, 0, light_value);
					return new GradientPaint(0, 0, dark, 10, 0, light, true);
				} else
					return new Color(1f, 0, 0);
			}
		}

	}

	private final static class EdgeWeightStrokeFunction implements
			EdgeStrokeFunction {
		protected static final Stroke basic = new BasicStroke(1);
		protected static final Stroke heavy = new BasicStroke(2);
		protected static final Stroke dotted = PluggableRenderer.DOTTED;

		protected boolean weighted = false;
		protected NumberEdgeValue edge_weight;

		public EdgeWeightStrokeFunction(NumberEdgeValue edge_weight) {
			this.edge_weight = edge_weight;
		}

		public void setWeighted(boolean weighted) {
			this.weighted = weighted;
		}

		public Stroke getStroke(Edge e) {
			if (weighted) {
				if (drawHeavy(e))
					return heavy;
				else
					return dotted;
			} else
				return basic;
		}

		protected boolean drawHeavy(Edge e) {
			double value = edge_weight.getNumber(e).doubleValue();
			if (value > 0.7)
				return true;
			else
				return false;
		}

	}

	private final static class VertexStrokeHighlight implements
			VertexStrokeFunction {
		protected boolean highlight = false;
		protected Stroke heavy = new BasicStroke(5);
		protected Stroke medium = new BasicStroke(3);
		protected Stroke light = new BasicStroke(1);
		protected PickedInfo pi;

		public VertexStrokeHighlight(PickedInfo pi) {
			this.pi = pi;
		}

		public void setHighlight(boolean highlight) {
			this.highlight = highlight;
		}

		public Stroke getStroke(Vertex v) {
			if (highlight) {
				if (pi.isPicked(v))
					return heavy;
				else {
					for (Iterator<?> iter = v.getNeighbors().iterator(); iter
							.hasNext();) {
						Vertex w = (Vertex) iter.next();
						if (pi.isPicked(w))
							return medium;
					}
					return light;
				}
			} else
				return light;
		}
	}

	private final static class FontHandler implements VertexFontFunction,
			EdgeFontFunction {
		protected boolean bold = false;
		Font f = new Font("Helvetica", Font.PLAIN, 12);
		Font b = new Font("Helvetica", Font.BOLD, 12);

		public void setBold(boolean bold) {
			this.bold = bold;
		}

		public Font getFont(Vertex v) {
			if (bold)
				return b;
			else
				return f;
		}

		public Font getFont(Edge e) {
			if (bold)
				return b;
			else
				return f;
		}
	}

	private final static class DirectionDisplayPredicate implements Predicate {
		protected boolean show_d;
		protected boolean show_u;

		public DirectionDisplayPredicate(boolean show_d, boolean show_u) {
			this.show_d = show_d;
			this.show_u = show_u;
		}

		public void showDirected(boolean b) {
			show_d = b;
		}

		public void showUndirected(boolean b) {
			show_u = b;
		}

		public boolean evaluate(Object arg0) {
			if (arg0 instanceof DirectedEdge && show_d)
				return true;
			if (arg0 instanceof UndirectedEdge && show_u)
				return true;
			return false;
		}
	}

	private final static class VertexDisplayPredicate implements Predicate {
		protected boolean filter_small;
		protected final static int MIN_DEGREE = 4;

		public VertexDisplayPredicate(boolean filter) {
			this.filter_small = filter;
		}

		public void filterSmall(boolean b) {
			filter_small = b;
		}

		public boolean evaluate(Object arg0) {
			Vertex v = (Vertex) arg0;
			if (filter_small)
				return (v.degree() >= MIN_DEGREE);
			else
				return true;
		}
	}

	/**
	 * Controls the shape, size, and aspect ratio for each vertex.
	 * 
	 * @author Joshua O'Madadhain
	 */
	private final static class VertexShapeSizeAspect extends
			AbstractVertexShapeFunction implements VertexSizeFunction,
			VertexAspectRatioFunction {
		protected boolean stretch = false;
		protected boolean scale = false;
		protected boolean funny_shapes = false;
		protected NumberVertexValue voltages;

		public VertexShapeSizeAspect(NumberVertexValue voltages) {
			this.voltages = voltages;
			setSizeFunction(this);
			setAspectRatioFunction(this);
		}

		public void setStretching(boolean stretch) {
			this.stretch = stretch;
		}

		public void setScaling(boolean scale) {
			this.scale = scale;
		}

		public void useFunnyShapes(boolean use) {
			this.funny_shapes = use;
		}

		public int getSize(Vertex v) {
			return 20;
			// if (scale)
			// return (int)(voltages.getNumber(v).doubleValue() * 30) + 20;
			// else
			// return 20;
		}

		public float getAspectRatio(Vertex v) {
			if (stretch)
				return (float) (v.inDegree() + 1) / (v.outDegree() + 1);
			else
				return 1.0f;
		}

		public Shape getShape(Vertex v) {
			if (funny_shapes) {
				if (v.degree() < 5) {
					int sides = Math.max(v.degree(), 3);
					return factory.getRegularPolygon(v, sides);
				} else
					return factory.getRegularStar(v, v.degree());
			} else
				return factory.getEllipse(v);
		}

	}

	/**
	 * a GraphMousePlugin that offers popup menu support
	 */
	protected class PopupGraphMousePlugin extends AbstractPopupGraphMousePlugin
			implements MouseListener {

		public PopupGraphMousePlugin() {
			this(MouseEvent.BUTTON3_MASK);
		}

		public PopupGraphMousePlugin(int modifiers) {
			super(modifiers);
		}

		/**
		 * If this event is over a Vertex, pop up a menu to allow the user to
		 * increase/decrease the voltage attribute of this Vertex
		 * 
		 * @param e
		 */
		protected void handlePopup(MouseEvent e) {
			final VisualizationViewer vv = (VisualizationViewer) e.getSource();
			Point2D p = vv.inverseViewTransform(e.getPoint());

			PickSupport pickSupport = vv.getPickSupport();
			if (pickSupport != null) {
				final Vertex v = pickSupport.getVertex(p.getX(), p.getY());
				if (v != null) {
					JPopupMenu popup = new JPopupMenu();
					popup.add(new AbstractAction("Decrease Transparency") {
						public void actionPerformed(ActionEvent e) {
							MutableDouble value = (MutableDouble) transparency
									.getNumber(v);
							value.setDoubleValue(Math.min(1, value
									.doubleValue() + 0.1));
							vv.repaint();
						}
					});
					popup.add(new AbstractAction("Increase Transparency") {
						public void actionPerformed(ActionEvent e) {
							MutableDouble value = (MutableDouble) transparency
									.getNumber(v);
							value.setDoubleValue(Math.max(0, value
									.doubleValue() - 0.1));
							vv.repaint();
						}
					});
					popup.show(vv, e.getX(), e.getY());
				} else {
					final Edge edge = pickSupport.getEdge(p.getX(), p.getY());
					if (edge != null) {
						JPopupMenu popup = new JPopupMenu();
						popup.add(new AbstractAction(edge.toString()) {
							public void actionPerformed(ActionEvent e) {
								System.err.println("got " + edge);
							}
						});
						popup.show(vv, e.getX(), e.getY());

					}
				}
			}
		}
	}

	public class VoltageTips extends DefaultToolTipFunction {

		@Override
		public String getToolTipText(Vertex v) {
			return "Activation level:" + neuron_activation.getNumber(v);
		}

		@Override
		public String getToolTipText(Edge edge) {
			return edge.toString();
		}
	}

	public class GradientPickedEdgePaintFunction extends
			GradientEdgePaintFunction {
		private PickedInfo pi;
		private EdgePaintFunction defaultFunc;
		private final Predicate self_loop = SelfLoopEdgePredicate.getInstance();
		protected boolean fill_edge = false;

		public GradientPickedEdgePaintFunction(
				EdgePaintFunction defaultEdgePaintFunction, HasGraphLayout vv,
				LayoutTransformer transformer, PickedInfo pi) {
			super(Color.WHITE, Color.BLACK, vv, transformer);
			this.defaultFunc = defaultEdgePaintFunction;
			this.pi = pi;
		}

		public void useFill(boolean b) {
			fill_edge = b;
		}

		@Override
		public Paint getDrawPaint(Edge e) {
			if (gradient_level == GRADIENT_NONE) {
				return defaultFunc.getDrawPaint(e);
			} else {
				return super.getDrawPaint(e);
			}
		}

		@Override
		protected Color getColor2(Edge e) {
			return pi.isPicked(e) ? Color.CYAN : c2;
		}

		@Override
		public Paint getFillPaint(Edge e) {
			if (self_loop.evaluate(e) || !fill_edge)
				return null;
			else
				return getDrawPaint(e);
		}

	}

}
